'use strict';

const api = { 
    events: require('events'),
    linker: require('./linker'),
    pro: require('./protocol'),
}

const createGraphics = port => {
    const result = () => {}

    result.event = new api.events.EventEmitter();
    result.initialized = false;

    let outputPackage = [];

    const linker = api.linker.createLinker(port, (send, buffer, remote) => {
        const datagram = api.pro.parseInput(buffer);
        //console.log(buffer);
        if(datagram.class === 105) {
            send(api.pro.addChecksum(api.pro.makeFrame(Buffer.from('i'))));
            console.log('initialized');
            result.initialized = true;
            result.event.emit('initialized', remote);
        }
        else if(datagram.class === 101) {
            if(datagram.eventType === 256) {
                result.event.emit('exit');
                send(api.pro.addChecksum(api.pro.makeFrame(Buffer.from('e'))));
                console.log('exit signal');
                setTimeout(process.exit, 100);
            }
            else if(datagram.eventType === 768) {
                result.event.emit('keydown', datagram.char);
            }
            else if(datagram.eventType === 771) {
                result.event.emit('key', datagram.char);
            }
            else if(datagram.eventType === 769) {
                result.event.emit('keyup', datagram.char);
            }
            else if(datagram.eventType === 1024) {
                result.event.emit('motion', datagram.mx, datagram.my);
            }
            else if(datagram.eventType === 1025) {
                result.event.emit('mousedown', datagram.mx, datagram.my);
            }
            else if(datagram.eventType === 1026) {
                result.event.emit('mouseup', datagram.mx, datagram.my);
            }
        }
    });
    
    result.pixel = (x, y) => {
        const buffer = api.pro.genBuffer(
            ['p', 1],
            [x, 4],
            [y, 4],
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.line = (x1, y1, x2, y2) => {
        const buffer = api.pro.genBuffer(
            ['l', 1],
            [x1, 4],
            [y1, 4],
            [x2, 4],
            [y2, 4]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.rect = (x1, y1, x2, y2) => {
        const buffer = api.pro.genBuffer(
            ['r', 1],
            [x1, 4],
            [y1, 4],
            [x2, 4],
            [y2, 4]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.square = (x, y, l) => {
        const buffer = api.pro.genBuffer(
            ['q', 1],
            [x, 4],
            [y, 4],
            [l, 4]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.circle = (x, y, r) => {
        const buffer = api.pro.genBuffer(
            ['d', 1],
            [x, 4],
            [y, 4],
            [r, 4]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.backdround = color => {
        const buffer = api.pro.genBuffer(
            ['b', 1],
            [color, 4, 'u']
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.setColor = color => {
        const buffer = api.pro.genBuffer(
            ['c', 1],
            [color, 4, 'u']
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.image = (x, y, key) => {
        const buffer = api.pro.genBuffer(
            ['g', 1],
            [x, 4],
            [y, 4],
            [key, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.imageRotated = (x, y, angle, key) => {
        const buffer = api.pro.genBuffer(
            ['j', 1],
            [x, 4],
            [y, 4],
            [angle, 4, 'f'], // not radians
            [key, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.loadImage = path => {
        const buffer = api.pro.genBuffer(
            ['h', 1],
            [path, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
        return path.split('/').pop().split('.')[0];
    }

    result.ttfText = (x, y, key) => {
        const buffer = api.pro.genBuffer(
            ['t', 1],
            [x, 4],
            [y, 4],
            [key, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.ttfLoadFont = (path, size) => {
        if(path === undefined) path = './node_modules/antihype/df.ttf';
        const buffer = api.pro.genBuffer(
            ['o', 1],
            [size ? size : 16, 4],
            [path, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.addToCache = data => {
        const buffer = api.pro.genBuffer(
            ['a', 1],
            [data, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.showCursor = condition => {
        const buffer = api.pro.genBuffer(
            ['s', 1],
            [condition, 1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.createWindow = (title, w, h, flags) => {
        result.window = { title, w, h, flags }
        const buffer = api.pro.genBuffer(
            ['w', 1],
            [w, 4,],
            [h, 4,],
            [flags, 4, 'u'],
            [title, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.resizeWindow = (x, y) => {
        const buffer = api.pro.genBuffer(
            ['y', 1],
            [x, 4],
            [y, 4]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.update = () => {
        const buffer = api.pro.genBuffer(
            ['u', 1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.exit = () => {
        const buffer = api.pro.genBuffer(
            ['e', 1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.applyPackage = () => {
        const datagram = api.pro.addChecksum(Buffer.concat(outputPackage));
        linker.send(datagram);
        outputPackage = [];
    }
    
    result.cacheLoadImage = key => {
        const buffer = api.pro.genBuffer(
            ['#', 1],
            [0, 4, 'u'],
            [key, -1]
        );
        const frame = api.pro.makeFrame(buffer);
        outputPackage.push(frame);
    }

    result.cacheLoadComposition = (key, x, y) => {
        const buffer = api.pro.genBuffer(
            ['#', 1],
            [2, 4, 'u'],
            [x, 4],
            [y, 4],
            [key, -1]
        );
        const frame = api.pro.makeFrame(buffer); 
        outputPackage.push(frame);
    }

    result.manual = () => {
        console.log({
            manual: {
                events: {
                    usage: 'graphics.event.on(e, cb (...args) => {})',
                    signals: [
                        {
                            sig: 'initialized',
                            args: ['remote']
                        },
                        {
                            sig: 'exit',
                            args: []
                        },
                        {
                            sig: 'keydown',
                            args: ['datagram.char']
                        },
                        {
                            sig: 'key',
                            args: ['datagram.char']
                        },
                        {
                            sig: 'keyup',
                            args: ['datagram.char']
                        },
                        {
                            sig: 'motion',
                            args: ['datagram.mx', 'datagram.my']
                        },
                        {
                            sig: 'mousedown',
                            args: ['datagram.mx', 'datagram.my']
                        },
                        {
                            sig: 'mouseup',
                            args: ['datagram.mx', 'datagram.my']
                        },
                    ]
                },
                examples: [
                    {
                        name: 'init',
                        code: () => {
                            const graphics = api.ah.createGraphics(7677);
                            graphics.event.on('initialized', remote => {
                                graphics.createWindow('window', 1920 / 2, 1080 / 2, 0);
                                graphics.showCursor(false);
                                graphics.ttfLoadFont('data/fonts/opensans-bold.ttf');
                                graphics.applyPackage();
                            });
                            
                            setInterval(()=> {
                                if(graphics.window) {
                                    graphics.backdround(0xC97878);
                                    
                                    graphics.update();
                                    graphics.applyPackage();
                                }
                            }, 40);
                        }
                    }
                ]
            }
        });
    }


    result.SDL_WINDOW_FULLSCREEN = 1;
    result.SDL_WINDOW_FULLSCREEN_DESKTOP = 4097;
    result.SDL_WINDOW_BORDERLESS = 16;
    result.SDL_WINDOW_OPENGL = 2;
    result.SDL_WINDOW_RESIZABLE = 32;

    return result;
}

module.exports = { createGraphics };
