'use strict';

const genBuffer = (...args) => {
    let sum = 0;
    for (const item of args) {
        if(item[1] >= 0) sum += item[1];
        else sum += item[0].length;
    };
    const result = Buffer.alloc(sum);
    let iterator = 0;
    for (const item of args) {
        if (item[1] !== 0) {
            if (item[1] === 1) {
                result.fill(item[0], iterator, iterator + 1);
            } else if (item[1] === 4) {
                if (item[2] === 'u') result.writeUInt32BE(item[0], iterator);
                else if (item[2] === 'f') result.writeFloatBE(item[0], iterator);
                else result.writeInt32BE(item[0], iterator);
            } else {
                result.write(item[0], iterator);
            }
            iterator += item[1];
        }
    }
    return result;
};

const addChecksum = buffer => {
    const add8 = (val1, val2) => {
        let result = val1 + val2;
        result %= 256;
        return result;
    };
    let checksum = 0;
    for (const c of buffer) {
        checksum = add8(checksum, c);
    }
    if (checksum > 255) console.log('checksum > 255');
    return Buffer.concat([buffer, Buffer.from([256 - checksum])]);
};

const makeFrame = buffer => {
    const frame = [];
    for (let c of buffer) {
        if (c === 0x7E || c === 0x7D) {
            frame.push(0x7D);
            c ^= 0x40;
        }
        frame.push(c);
    }
    frame.push(0x7E);
    return Buffer.from(frame);
};

const parseInput = buffer => {
    const result = {};
    //console.log(buffer);
    const frames = Math.floor(buffer.length / 4);
    if (frames > 0) result.class = buffer.readUInt32BE(0);
    if (frames > 1) result.eventType = buffer.readUInt32BE(4);
    if (frames > 2) result.char = buffer.readUInt32BE(8);
    if (frames > 3) result.mx = buffer.readInt32BE(12);
    if (frames > 4) result.my = buffer.readInt32BE(16);
    return result;
};

module.exports = {
    genBuffer,
    addChecksum,
    makeFrame,
    makeDatagram: buffer => addChecksum(makeFrame(buffer)),
    parseInput,
};
